import org.antlr.v4.runtime.*;
import org.antlr.v4.runtime.tree.ParseTree;

import java.io.FileInputStream;
import java.io.InputStream;

import antlr4.*;

import frontend.*;
import intermediate.util.BackendMode;

import static intermediate.util.BackendMode.*;

public class Pcl6 
{
    public static void main(String[] args) throws Exception 
    {
        if (args.length != 2)
        {
            System.out.println("USAGE: Pcl6 option sourceFileName");
            System.out.println("   option: -convert, -execute, or -compile");
            return;
        }
        
        String option = args[0];
        String sourceFileName = args[1];

        BackendMode mode = EXECUTOR;
        
        if      (option.equalsIgnoreCase("-convert")) mode = CONVERTER;
        else if (option.equalsIgnoreCase("-execute")) mode = EXECUTOR;
        else if (option.equalsIgnoreCase("-compile")) mode = COMPILER;
        else
        {
            System.out.println("ERROR: Invalid option.");
            System.out.println("   Valid options: -convert, -execute, or -compile");
        }
        
        // Generate a source file listing.
        new Listing(sourceFileName);
        
        // Create the input stream.
        InputStream source = new FileInputStream(sourceFileName);
        
        // Create the character stream from the input stream.
        CharStream cs = CharStreams.fromStream(source);
        
        // Custom syntax error handler.
        SyntaxErrorHandler syntaxErrorHandler = new SyntaxErrorHandler();
        
        // Create a lexer which scans the character stream
        // to create a token stream.
        Pcl6Lexer lexer = new Pcl6Lexer(cs);
        lexer.removeErrorListeners();
        lexer.addErrorListener(syntaxErrorHandler);
        CommonTokenStream tokens = new CommonTokenStream(lexer);
        
        // Create a parser which parses the token stream.
        Pcl6Parser parser = new Pcl6Parser(tokens);
        
        // Pass 1: Check syntax and create the parse tree.
        System.out.printf("\nPASS 1 Syntax: ");
        parser.removeErrorListeners();
        parser.addErrorListener(syntaxErrorHandler);
        ParseTree tree = parser.program();
        
        int errorCount = syntaxErrorHandler.getCount();
        if (errorCount > 0) 
        {
            System.out.printf("\nThere were %d syntax errors.\n", errorCount);
            System.out.println("Object file not created or modified.");
            return;
        }
        else
        {
            System.out.println("There were no syntax errors.");
        }
        
        // Pass 2: Semantic operations.
        System.out.printf("\nPASS 2 Semantics:\n");
        Semantics pass2 = new Semantics(mode);
        pass2.visit(tree);
        
        errorCount = pass2.getErrorCount();
        if (errorCount > 0)
        {
            System.out.printf("\nThere were %d semantic errors.\n", errorCount);
            System.out.println("Object file not created or modified.");
            return;
        }

        // Pass 3: Translation.
        switch (mode)
        {
            case CONVERTER:
            {
                break;
            }
                
            case EXECUTOR:
            {
                break;
            }
            
            case COMPILER:
            {
                break;
            }
        }
    }
}
